﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.ComponentModel;
using System.Collections.ObjectModel;
using Infragistics.Windows.DataPresenter;
using System.Diagnostics;

namespace XamDataGridActiveItem
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();

            this.DataContext = new ViewModel();
        }

        private void GroupByRecordPresenter_Loaded(object sender, RoutedEventArgs e)
        {
            (sender as GroupByRecordPresenter).Record.Description = (sender as GroupByRecordPresenter).Record.Description;
        }
    }

    #region Person
    public class Person //: INotifyPropertyChanged
    {
        private int id;

        public int Id
        {
            get { return id; }
            set
            {
                id = value;
                //NotifyPropertyChanged("Id");
            }
        }

        private int age;

        public int Age
        {
            get { return age; }
            set
            {
                age = value;
                //NotifyPropertyChanged("Age"); 
            }
        }

        private String lastName;

        public String LastName
        {
            get { return lastName; }
            set
            {
                lastName = value;
                //NotifyPropertyChanged("LastName"); 
            }
        }

        //public event PropertyChangedEventHandler PropertyChanged;

        //private void NotifyPropertyChanged(String info)
        //{
        //    if (PropertyChanged != null)
        //    {
        //        PropertyChanged(this, new PropertyChangedEventArgs(info));
        //    }

        //}

    }
    #endregion
   
    public class ViewModel : INotifyPropertyChanged
    {
        public ObservableCollection<Person> Data { get; set; }

        private Record _activeItem;
        public Record ActiveItem
        {
            get
            {
                return _activeItem;
            }
            set
            {
                _activeItem = value;
            }
        }

        RelayCommand _addCommand;
        public ICommand AddCommand
        {
            get
            {
                if (_addCommand == null)
                {
                    _addCommand = new RelayCommand(param => this.Add(), param => this.CanAdd);
                }
                return _addCommand;
            }
        }
        public void Add()
        {
            Person newPerson = new Person() { Id = 15, Age = 12, LastName = "LastName 2" };
            this.Data.Add(newPerson);

            this.collectionView.View.Refresh();
        }
        public bool CanAdd
        {
            get
            {
                return true;
            }
        }

        RelayCommand _removeCommand;
        public ICommand RemoveCommand
        {
            get
            {
                if (_removeCommand == null)
                {
                    _removeCommand = new RelayCommand(param => this.Remove(), param => this.CanRemove);
                }
                return _removeCommand;
            }
        }
        public void Remove()
        {
            Data.RemoveAt(Data.IndexOf((this.ActiveItem as DataRecord).DataItem as Person));
            
            this.collectionView.View.Refresh();
        }
        public bool CanRemove
        {
            get
            {
                return (this.ActiveItem != null) && (this.ActiveItem is DataRecord);
            }
        }

        public ViewModel()
        {
            Data = new ObservableCollection<Person>();
            Data.Add(new Person{Id = 1,
                Age = 24,
                LastName = "LastName 1"
            });
            Data.Add(new Person
            {
                Id = 2,
                Age = 34,
                LastName = "LastName 2"
            });
            Data.Add(new Person
            {
                Id = 3,
                Age = 32343,
                LastName = "LastName 4"
            });
            Data.Add(new Person
            {
                Id = 4,
                Age = 122,
                LastName = "LastName 1"
            });
            Data.Add(new Person
            {
                Id = 5,
                Age = 234,
                LastName = "LastName 2"
            });
        }

        private CollectionViewSource collectionView;
        public CollectionViewSource CollectionView
        {
            get
            {
                if (collectionView == null)
                {
                    collectionView = new CollectionViewSource();
                    collectionView.Filter += new FilterEventHandler(_ViewSource_Filter);

                    var groupDescription = new PropertyGroupDescription("LastName");
                    groupDescription.GroupNames.Add("LastName 1");
                    groupDescription.GroupNames.Add("LastName 2");
                    groupDescription.GroupNames.Add("LastName 3");

                    collectionView.GroupDescriptions.Add(groupDescription);
                    collectionView.Source = this.Data;
                }

                return collectionView;
            }
        }

        private string _FilterString;
        public string FilterString
        {
            get
            {
                return _FilterString;
            }
            set
            {
                _FilterString = value;
                NotifyPropertyChanged(this.FilterString);

                CollectionView.View.Refresh();
            }
        }

        void _ViewSource_Filter(object sender, FilterEventArgs e)
        {
            Person r = e.Item as Person;

            if (r == null) e.Accepted = false;


            if (string.IsNullOrEmpty(FilterString))
            {
                e.Accepted = true;
            }
            else
            {
                if (r.LastName.ToUpper().Contains(FilterString.ToUpper().Trim()))
                    e.Accepted = true;
                else
                    e.Accepted = false;
            }

        }

        public event PropertyChangedEventHandler PropertyChanged;
        private void NotifyPropertyChanged(String info)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(info));
            }

        }

    }

    public class RelayCommand : ICommand
    {
        #region Fields

        readonly Action<object> _execute;
        readonly Predicate<object> _canExecute;

        #endregion // Fields

        #region Constructors

        public RelayCommand(Action<object> execute)
            : this(execute, null)
        {
        }

        public RelayCommand(Action<object> execute, Predicate<object> canExecute)
        {
            if (execute == null)
                throw new ArgumentNullException("execute");

            _execute = execute;
            _canExecute = canExecute;
        }
        #endregion // Constructors

        #region ICommand Members

        [DebuggerStepThrough]
        public bool CanExecute(object parameter)
        {
            return _canExecute == null ? true : _canExecute(parameter);
        }

        public event EventHandler CanExecuteChanged
        {
            add { CommandManager.RequerySuggested += value; }
            remove { CommandManager.RequerySuggested -= value; }
        }

        public void Execute(object parameter)
        {
            _execute(parameter);
        }

        #endregion // ICommand Members
    }
}
